@@ -8,10 +8,10 @@
  * 
  *          Main layer of quota management
  * 
- * Version: $Id: dquot.c,v 3.7 1994/05/30 18:38:18 mvw Exp mvw $
+ * Version: $Id: dquot.c,v 3.11 1994/10/17 09:52:47 mvw Exp mvw $
  * 
- * Authors: Marco van Wieringen <v892273@si.hhs.nl> <mvw@mcs.nl.mugnet.org>
- *          Edvard Tuinder <v892231@si.hhs.nl> <ed@delirium.nl.mugnet.org>
+ * Authors: Marco van Wieringen <v892273@si.hhs.nl> <mvw@mcs.ow.org>
+ *          Edvard Tuinder <v892231@si.hhs.nl> <ed@delirium.ow.org>
  *          Linus Torvalds <Linus.Torvalds@cc.helsinki.FI>
  * 
  *          This program is free software; you can redistribute it and/or
@@ -35,12 +35,11 @@
 #include <linux/malloc.h>
 #include <linux/mount.h>
 #include <linux/fileio.h>
+#include <linux/delay.h>
 
 #include <asm/segment.h>
 #include <sys/sysmacros.h>
 
-#define DEBUG_QUOTA 0 /* Level of debugging the higher the more stuff you get */
-
 static char quotamessage[MAX_QUOTA_MESSAGE];
 static char *quotatypes[] = INITQFNAMES;
 
@@ -211,18 +210,22 @@ static void write_dquot(struct dquot *dquot)
 
    if (!(dquot->dq_flags & DQ_MOD) || (filp == (struct file *)0))
       return;
+   lock_dquot(dquot);
+   down(&dquot->dq_mnt->mnt_sem);
    if (filp->f_op->lseek) {
       if (filp->f_op->lseek(filp->f_inode, filp,
-                            dqoff(dquot->dq_id),0) != dqoff(dquot->dq_id))
+          dqoff(dquot->dq_id), 0) != dqoff(dquot->dq_id)) {
+         up(&dquot->dq_mnt->mnt_sem);
          return;
+      }
    } else
       filp->f_pos = dqoff(dquot->dq_id);
-   lock_dquot(dquot);
    fs = get_fs();
    set_fs(KERNEL_DS);
-   if (filp->f_op->write(filp->f_inode, filp, (char *)&dquot->dq_dqb,
-                         sizeof(struct dqblk)) == sizeof(struct dqblk))
+   if (filp->f_op->write(filp->f_inode, filp,
+       (char *)&dquot->dq_dqb, sizeof(struct dqblk)) == sizeof(struct dqblk))
       dquot->dq_flags &= ~DQ_MOD;
+   up(&dquot->dq_mnt->mnt_sem);
    set_fs(fs);
    unlock_dquot(dquot);
 }
@@ -235,17 +238,21 @@ static void read_dquot(struct dquot *dquot)
 
    if (filp == (struct file *)0)
       return;
+   lock_dquot(dquot);
+   down(&dquot->dq_mnt->mnt_sem);
    if (filp->f_op->lseek) {
       if (filp->f_op->lseek(filp->f_inode, filp,
-                            dqoff(dquot->dq_id),0) != dqoff(dquot->dq_id))
+          dqoff(dquot->dq_id),0) != dqoff(dquot->dq_id)) {
+         up(&dquot->dq_mnt->mnt_sem);
          return;
+      }
    } else
       filp->f_pos = dqoff(dquot->dq_id);
-   lock_dquot(dquot);
    fs = get_fs();
    set_fs(KERNEL_DS);
    filp->f_op->read(filp->f_inode, filp,
                     (char *)&dquot->dq_dqb, sizeof(struct dqblk));
+   up(&dquot->dq_mnt->mnt_sem);
    set_fs(fs);
    if (dquot->dq_bhardlimit == 0 && dquot->dq_bsoftlimit == 0 &&
        dquot->dq_ihardlimit == 0 && dquot->dq_isoftlimit == 0)
@@ -356,7 +363,7 @@ static int check_bdq(struct dquot *dquot, int id, short type,
          tty_write_message(current->tty, quotamessage);
          dquot->dq_flags |= DQ_BLKS;
       }
-      if (dquot->dq_curblocks != dquot->dq_bhardlimit) {
+      if (dquot->dq_curblocks < dquot->dq_bhardlimit) {
          *avail_blocks = dquot->dq_bhardlimit - dquot->dq_curblocks;
          return QUOTA_OK;
       } else
@@ -501,9 +508,8 @@ static int get_quota(dev_t dev, int id, short type, struct dqblk * dqblk)
          dqput(dquot);
          return 0;
       }
-      return -ESRCH;
-   } else
-      return -ESRCH;
+   }
+   return -ESRCH;
 }
 
 /*
@@ -514,30 +520,19 @@ int quota_off(dev_t dev, short type)
    struct vfsmount *vfsmnt;
    short cnt;
 
-   if (type != -1) {
+   for (cnt = 0; cnt < MAXQUOTAS; cnt++) {
+      if (type != -1 && cnt != type)
+         continue;
       if ((vfsmnt = lookup_vfsmnt(dev)) == (struct vfsmount *)0 ||
-           vfsmnt->mnt_quotas[type] == (struct file *)0)
-         return -ESRCH;
+           vfsmnt->mnt_quotas[cnt] == (struct file *)0)
+         continue;
       vfsmnt->mnt_flags |= QF_CLOSING;
-      reset_dquot_ptrs(dev, type);
-      invalidate_dquots(dev, type);
-      close_fp(vfsmnt->mnt_quotas[type], 0);
-      vfsmnt->mnt_quotas[type] = (struct file *)0;
-      vfsmnt->mnt_iexp[type] = vfsmnt->mnt_bexp[type] = (time_t)0;
+      reset_dquot_ptrs(dev, cnt);
+      invalidate_dquots(dev, cnt);
+      close_fp(vfsmnt->mnt_quotas[cnt], 0);
+      vfsmnt->mnt_quotas[cnt] = (struct file *)0;
+      vfsmnt->mnt_iexp[cnt] = vfsmnt->mnt_bexp[cnt] = (time_t)0;
       vfsmnt->mnt_flags &= ~QF_CLOSING;
-   } else {
-      for (cnt = 0; cnt < MAXQUOTAS; cnt++) {
-         if ((vfsmnt = lookup_vfsmnt(dev)) == (struct vfsmount *)0 ||
-              vfsmnt->mnt_quotas[cnt] == (struct file *)0)
-            continue;
-         vfsmnt->mnt_flags |= QF_CLOSING;
-         reset_dquot_ptrs(dev, cnt);
-         invalidate_dquots(dev, cnt);
-         close_fp(vfsmnt->mnt_quotas[cnt], 0);
-         vfsmnt->mnt_quotas[cnt] = (struct file *)0;
-         vfsmnt->mnt_iexp[cnt] = vfsmnt->mnt_bexp[cnt] = (time_t)0;
-         vfsmnt->mnt_flags &= ~QF_CLOSING;
-      }
    }
    return 0;
 }
@@ -560,9 +555,12 @@ static int quota_on(dev_t dev, short type, char *path)
    vfsmnt->mnt_flags |= QF_OPENING;
    error = open_namei(tmp, O_RDWR, 0600, &inode, 0);
    putname(tmp);
-   if (error)
+   if (error) {
+      vfsmnt->mnt_flags &= ~QF_OPENING;
       return (error);
+   }
    if (!S_ISREG(inode->i_mode)) {
+      vfsmnt->mnt_flags &= ~QF_OPENING;
       iput(inode);
       return -EACCES;
    }
@@ -591,6 +589,7 @@ static int quota_on(dev_t dev, short type, char *path)
    add_dquot_ref(dev, type);
    return 0;
 end_quotaon:
+   vfsmnt->mnt_flags &= ~QF_OPENING;
    if (filp != (struct file *)0)
       filp->f_count--;
    iput(inode);
@@ -610,12 +609,6 @@ void dqput(struct dquot *dquot)
     * dquot that is put back into the freelist.
     */
    if (dquot->dq_mnt != (struct vfsmount *)0) {
-#if DEBUG_QUOTA > 0
-      printk("VFS: dqput for %s %d, on dev (%d,%d)\n",
-             quotatypes[dquot->dq_type], dquot->dq_id,
-             MAJOR(dquot->dq_dev), MINOR(dquot->dq_dev));
-      printk("VFS: New count will be %d\n", dquot->dq_count - 1);
-#endif
       wait_on_dquot(dquot);
       if (!dquot->dq_count) {
          printk("VFS: iput: trying to free free dquot\n");
@@ -671,7 +664,7 @@ repeat:
       }
    dquot = best;
    if (!dquot) {
-      printk("VFS: No free dquots - contact mvw@mcs.nl.mugnet.org\n");
+      printk("VFS: No free dquots - contact mvw@mcs.ow.org\n");
       sleep_on(&dquot_wait);
       goto repeat;
    }
@@ -697,7 +690,7 @@ repeat:
 
 /*
  * Just like iget, increment referencecount of a dquot and return
- * pointer to it in the hashqueueu.
+ * pointer to it in the hashqueue.
  */
 struct dquot *dqget(dev_t dev, unsigned int id, short type)
 {
@@ -708,10 +701,6 @@ struct dquot *dqget(dev_t dev, unsigned int id, short type)
        (vfsmnt->mnt_flags & (QF_OPENING | QF_CLOSING)) ||
        (vfsmnt->mnt_quotas[type] == (struct file *)0))
       return (NODQUOT);
-#if DEBUG_QUOTA > 0
-   printk("VFS: dqget for %s %d, on dev (%d,%d)\n",
-          quotatypes[type], id, MAJOR(dev), MINOR(dev));
-#endif
    empty = get_empty_dquot();
 repeat:
    dquot = *(hash(dev, id, type));
@@ -756,10 +745,6 @@ void getinoquota(struct inode *inode, short type)
       if (type != -1 && cnt != type)
          continue;
       if (inode->i_dquot[cnt] != NODQUOT) {
-#if DEBUG_QUOTA > 0
-         printk("VFS: Increment already initialized dquot ptr for %s %d\n",
-                quotatypes[cnt], inode->i_dquot[cnt]->dq_id);
-#endif
          inode->i_dquot[cnt]->dq_count++;
          continue;
       }
@@ -842,11 +827,6 @@ int quota_alloc(struct inode *inode, u_long wanted_inodes, u_long wanted_blocks,
    u_long availblocks, local_avail;
    unsigned short cnt;
 
-#if DEBUG_QUOTA > 1
-   printk("VFS: quota_alloc for dev (%d,%d), uid %d, gid %d inodes %d, blocks %d\n",
-          MAJOR(inode->i_dev), MINOR(inode->i_dev), inode->i_uid, inode->i_gid,
-          wanted_inodes, wanted_blocks);
-#endif
    availblocks = wanted_blocks;
    if (wanted_inodes > 0 || wanted_blocks > 0) {
       for (cnt = 0; cnt < MAXQUOTAS; cnt++) {
@@ -878,11 +858,6 @@ void quota_remove(struct inode *inode, u_long inodes, u_long blocks)
 {
    short cnt;
 
-#if DEBUG_QUOTA > 1
-   printk("VFS: quota_remove for dev (%d,%d), uid %d, gid %d, ino %d, blocks %d\n", 
-          MAJOR(inode->i_dev), MINOR(inode->i_dev), inode->i_uid, inode->i_gid,
-          inodes, blocks);
-#endif
    if (inodes > 0 || blocks > 0) {
       for (cnt = 0; cnt < MAXQUOTAS; cnt++) {
          if (inode->i_dquot[cnt] == NODQUOT)
@@ -903,12 +878,6 @@ int quota_transfer(struct inode *inode, uid_t newuid, gid_t newgid,
    unsigned int id = 0;
    short cnt, disc;
 
-#if DEBUG_QUOTA > 1
-   printk("VFS: quota_transfer for dev (%d,%d), uid %d to %d, gid %d to %d, ino %d, blocks %d\n", 
-          MAJOR(inode->i_dev), MINOR(inode->i_dev), inode->i_uid, newuid,
-          inode->i_gid, newgid, inodes, blocks);
-#endif
-
    if (inodes > 0 || blocks > 0) {
       for (cnt = 0; cnt < MAXQUOTAS; cnt++) {
          transfer_to[cnt] = NODQUOT;
