@@ -25,7 +25,7 @@
 #include <asm/system.h>
 #include <asm/segment.h>
 #include <asm/bitops.h>
- 
+
 extern struct file_operations * get_blkfops(unsigned int);
 
 extern void wait_for_keypress(void);
@@ -41,97 +41,97 @@ static int do_remount_sb(struct super_block *sb, int flags, char * data);
 dev_t ROOT_DEV = 0;
 
 struct super_block super_blocks[NR_SUPER];
-static struct file_system_type * file_systems = (struct file_system_type *) NULL;
+static struct file_system_type *file_systems = (struct file_system_type *) NULL;
 static struct vfsmount *vfsmntlist = (struct vfsmount *) NULL,
                        *vfsmnttail = (struct vfsmount *) NULL,
                        *mru_vfsmnt = (struct vfsmount *) NULL;
 
-/*
+/* 
  * This part handles the management of the list of mounted filesystems.
  * Superblock retrieval is also done with this list.
  */
 struct vfsmount *lookup_vfsmnt(dev_t dev)
 {
-      register struct vfsmount *lptr;
+	register struct vfsmount *lptr;
 
-      if (vfsmntlist == (struct vfsmount *) 0)
-              return ((struct vfsmount *) 0);
+	if (vfsmntlist == (struct vfsmount *) 0)
+		return ((struct vfsmount *) 0);
 
-      if (mru_vfsmnt != (struct vfsmount *) 0 && mru_vfsmnt->mnt_dev == dev)
-              return (mru_vfsmnt);
+	if (mru_vfsmnt != (struct vfsmount *) 0 && mru_vfsmnt->mnt_dev == dev)
+		return (mru_vfsmnt);
 
-      for (lptr = vfsmntlist; lptr != (struct vfsmount *)0; lptr = lptr->mnt_next) 
-              if (lptr->mnt_dev == dev)
-                      return (lptr);
+	for (lptr = vfsmntlist; lptr != (struct vfsmount *)0; lptr = lptr->mnt_next)
+		if (lptr->mnt_dev == dev)
+			return (lptr);
 
-      return ((struct vfsmount *) 0);
-      /* NOTREACHED */
+	return ((struct vfsmount *) 0);
+	/* NOTREACHED */
 }
 
 static struct vfsmount *add_vfsmnt(dev_t dev, const char *dev_name,
-                                 const char *dir_name)
+				   const char *dir_name)
 {
-      register struct vfsmount *lptr;
-      char *tmp;
+	register struct vfsmount *lptr;
+	char *tmp;
 
-      if ((lptr = (struct vfsmount *)
-           kmalloc(sizeof(struct vfsmount), GFP_KERNEL)) == (struct vfsmount *) 0)
-              panic("VFS: Unable to allocate memory for vfsmount devicelist");
-      memset(lptr, 0, sizeof(struct vfsmount));
-      lptr->mnt_dev = dev;
-      if (dev_name) {
-              if (!getname(dev_name, &tmp)) {
-                      if ((lptr->mnt_devname =
-                          (char *) kmalloc(strlen(tmp), GFP_KERNEL)) != (char *)0)
-                              strcpy(lptr->mnt_devname, tmp);
-                      putname(tmp);
-              }
-      }
-      if (dir_name) {
-              if (!getname(dir_name, &tmp)) {
-                      if ((lptr->mnt_dirname =
-                          (char *) kmalloc(strlen(tmp), GFP_KERNEL)) != (char *)0)
-                      strcpy(lptr->mnt_dirname, tmp);
-                      putname(tmp);
-              }
-      }
+	if ((lptr = (struct vfsmount *)
+	     kmalloc(sizeof(struct vfsmount), GFP_KERNEL)) == (struct vfsmount *) 0)
+		panic("VFS: Unable to allocate memory for vfsmount devicelist");
 
-      if (vfsmntlist == (struct vfsmount *)0)
-              vfsmntlist = vfsmnttail = lptr;
-      else {
-              vfsmnttail->mnt_next = lptr;
-              vfsmnttail = lptr;
-      }
-      return (lptr);
+	memset(lptr, 0, sizeof(struct vfsmount));
+	lptr->mnt_dev = dev;
+	if (dev_name) {
+		if (!getname(dev_name, &tmp)) {
+			if ((lptr->mnt_devname =
+			    (char *) kmalloc(strlen(tmp), GFP_KERNEL)) != (char *)0)
+				strcpy(lptr->mnt_devname, tmp);
+			putname(tmp);
+		}
+	}
+	if (dir_name) {
+		if (!getname(dir_name, &tmp)) {
+			if ((lptr->mnt_dirname =
+			    (char *) kmalloc(strlen(tmp), GFP_KERNEL)) != (char *)0)
+			strcpy(lptr->mnt_dirname, tmp);
+			putname(tmp);
+		}
+	}
+
+	if (vfsmntlist == (struct vfsmount *)0)
+		vfsmntlist = vfsmnttail = lptr;
+	else {
+		vfsmnttail->mnt_next = lptr;
+		vfsmnttail = lptr;
+	}
+	return (lptr);
 }
 
 static void remove_vfsmnt(dev_t dev)
 {
-      register struct vfsmount *lptr, *tofree;
+	register struct vfsmount *lptr, *tofree;
 
-      if (vfsmntlist == (struct vfsmount *) 0)
-              return;
-      lptr = vfsmntlist;
-      if (lptr->mnt_dev == dev) {
-              tofree = lptr;
-              vfsmntlist = lptr->mnt_next;
-      } else {
-              while (lptr->mnt_next != (struct vfsmount *) 0) {
-                      if (lptr->mnt_next->mnt_dev == dev)
-                              break;
-                      lptr = lptr->mnt_next;
-              }
-              tofree = lptr->mnt_next;
-              if (vfsmnttail->mnt_dev == dev)
-                      vfsmnttail = lptr;
-              lptr->mnt_next = lptr->mnt_next->mnt_next;
-      }
-      kfree(tofree->mnt_devname);
-      kfree(tofree->mnt_dirname);
-      kfree_s(tofree, sizeof(struct vfsmount));
+	if (vfsmntlist == (struct vfsmount *) 0)
+		return;
+	lptr = vfsmntlist;
+	if (lptr->mnt_dev == dev) {
+		tofree = lptr;
+		vfsmntlist = lptr->mnt_next;
+	} else {
+		while (lptr->mnt_next != (struct vfsmount *) 0) {
+			if (lptr->mnt_next->mnt_dev == dev)
+				break;
+			lptr = lptr->mnt_next;
+		}
+		tofree = lptr->mnt_next;
+		if (vfsmnttail->mnt_dev == dev)
+			vfsmnttail = lptr;
+		lptr->mnt_next = lptr->mnt_next->mnt_next;
+	}
+	kfree(tofree->mnt_devname);
+	kfree(tofree->mnt_dirname);
+	kfree_s(tofree, sizeof(struct vfsmount));
 }
 
-
 int register_filesystem(struct file_system_type * fs)
 {
 	struct file_system_type ** tmp;
@@ -576,6 +576,7 @@ static int do_mount(dev_t dev, const char * dev_name, const char * dir_name,
 	}
 	vfsmnt = add_vfsmnt(dev, dev_name, dir_name);
 	vfsmnt->mnt_sb = sb;
+	vfsmnt->mnt_sem.count = 1;
 	sb->s_covered = dir_i;
 	dir_i->i_mount = sb->s_mounted;
 	return 0;		/* we don't iput(dir_i) - see umount */
@@ -758,11 +759,11 @@ void mount_root(void)
 {
 	struct file_system_type * fs_type;
 	struct super_block * sb;
-        struct vfsmount *vfsmnt;
+	struct vfsmount *vfsmnt;
 	struct inode * inode, d_inode;
 	struct file filp;
 	int retval;
-
+  
 	memset(super_blocks, 0, sizeof(super_blocks));
 	fcntl_init_locks();
 	if (MAJOR(ROOT_DEV) == FLOPPY_MAJOR) {
@@ -788,9 +789,9 @@ void mount_root(void)
 	for (fs_type = file_systems ; fs_type ; fs_type = fs_type->next) {
 		if(retval)
 			break;
-		if (!fs_type->requires_dev)
-			continue;
-		sb = read_super(ROOT_DEV,fs_type->name,root_mountflags,NULL,1);
+  		if (!fs_type->requires_dev)
+  			continue;
+  		sb = read_super(ROOT_DEV,fs_type->name,root_mountflags,NULL,1);
 		if (sb) {
 			inode = sb->s_mounted;
 			inode->i_count += 3 ;	/* NOTE! it is logically used 4 times, not 1 */
@@ -803,6 +804,7 @@ void mount_root(void)
 				(sb->s_flags & MS_RDONLY) ? " readonly" : "");
 			vfsmnt = add_vfsmnt(ROOT_DEV, "rootfs", "/");
 			vfsmnt->mnt_sb = sb;
+			vfsmnt->mnt_sem.count = 1;
 			return;
 		}
 	}
